#pragma once

// Class:   CTVItemDataMgr 
// Purpose: Simple class that represents data contained in a tree control item.
// 
// Public Methods:
//      Cancel()        -- Cancels the retrieve and populate operations
//      Populate(...)   -- Populates a tree control
//      Retrieve(...)   -- Retrieves data (fills a list with data)
class CTVItemDataMgr
{
public:

   const static long MAX_ITEMLEN = MAX_PATH + 1;

    // Retrieval states
    enum { TVIDM_NOTRETRIEVED = 0, TVIDM_INPROGRESS = 1, TVIDM_RETRIEVED = 2 };
    
   CTVItemDataMgr( );
   ~CTVItemDataMgr( );

// Operations (public methods consumed by CVirtualTreeDlg)
public:
   // Applies the multiplier to the retrieved directory items
   void ApplyMultiplier( );
   // Cancels pending operations
   HRESULT Cancel();
   // Fills the tree control with CTVItemData items
   HRESULT Populate( CTreeCtrl& ctlTree, size_t& ruCount );

   // Retrieves the directory structure from the harddrive and inserts items
   // into the CTVItemData multimap
   HRESULT Retrieve( LPCTSTR szDriveLetter,
                     const LONG lMaxCount,
                     const LONG lMultiplier,
                     const LONG lMaxDepth );

// Implementation
private:
   // Deletes any CTVItemData within the multimap
   void    ClearTVItemDataMMap();

   BOOL AddFoldersRecursively( LPCTSTR szFolder, LPCTSTR szName, const LONG lParentID );


   // Recursive method
   HRESULT	InsertTreeNode( CTreeCtrl& ctrTree, 
                              const long lParentNode,
                              HTREEITEM hParentItem,
                              BOOL& bExpand );

   // Creates a new CLVItemData object and inserts it into the list
   HRESULT InsertItem( LPCTSTR szName, const LONG lParentID, LONG& lNewID );

   // Returns the Next ID (thread safe)
   const LONG GetNextID( );

   // Sets the ID to zero (thread safe)
   void ResetID( );

   // Returns the MaxCount (thread safe)
   const LONG GetMaxCount( );

   // Returns the multiplier (thread safe)
   const LONG GetMultiplier( );

   // Returns the depth (thread safe)
   const LONG GetMaxDepth( );

   // Returns the current retrieval state (thread safe)
   const UINT GetRetrievalState( );

   // Returns the shutdown event handle
   HANDLE& GetShutdownEvent( );

   //
   // Threading and Message Processing while waiting for threads
   //

   // Waits for the retrieve thread to finish
   HRESULT MsgWaitForThread( HANDLE hThread );

   // Processes windows messages
   BOOL ProcessMessages( );

   // Secondary Thread Proc which adds directory items to the TVItemData multi-map
   static UINT WINAPI RetrieveThread( LPVOID lpContext );

   // Method:  ShutdownRetrieveThread
   void ShutdownRetrieveThread( );

   //
   // Accessors
   //

   // Drive Letter (for retrieving directory structure) 
   LPCTSTR GetDriveLetter( );
   void SetDriveLetter( LPCTSTR szDriveLetter );
   
   // Max Count -- Limits total tree items
   void SetMaxCount( const LONG lMaxCount );

   // Multiplier 
   void SetMultiplier( const LONG lMultiplier );
  
   // Depth -- Limits number of child tree nodes
   const LONG  DecCurrentDepth( );
   const LONG  GetCurrentDepth( );   
   const LONG  IncCurrentDepth( );
   void        SetMaxDepth( const LONG lMaxDepth );
   
   // Retrieval State (one of TVIDM_NOTRETRIEVED, TVIDM_INPROGRESS, or TVIDM_RETRIEVED)
   void SetRetrievalState( const LONG lState );
   

// Attributes
private:
   HANDLE            m_hShutdownEvent;
   HANDLE            m_hRetrieveThread;
   CTVItemDataMMap   m_TVItemDataMMap;
   LONG              m_lMaxCount;
   LONG              m_lMultiplier;
   LONG              m_lMaxDepth;
   LONG              m_lRetrievalState;
   LONG              m_lID;
   CString           m_sDriveLetter;
   CLockableCS       m_csDriveLetter;
   LONG              m_lCurrentDepth;
};


